import * as Joi from 'joi';
import { MiddlewareConfig } from './middleware';
/**
 * Configure Authentication for APIs.
 */
export interface AuthenticationConfig {
    /**
     * The strategy used for authentication
     */
    strategy: MiddlewareConfig;
}
export interface ApiAuthenticationConfig extends AuthenticationConfig {
    /**
     * A list of groups that should be handled by this authenticator. If not provided, everything
     * will be handled.
     * Defaults to *.
     */
    group?: Array<string>;
    /**
     * Import a configuration from gateway config session
     */
    use?: string;
}
export interface BasicAuthentication {
    /**
     * Is a function with the parameters verify(userid, password, done) {
     *  - userid The username.
     *  - password The password.
     *  - done is a passport error first callback accepting arguments done(error, user, info)
     */
    verify: MiddlewareConfig;
}
export interface LocalAuthentication {
    /**
     * Is a function with the parameters verify(userid, password, done) {
     *  - userid The username.
     *  - password The password.
     *  - done is a passport error first callback accepting arguments done(error, user, info)
     */
    verify: MiddlewareConfig;
    /**
     * Optional, defaults to 'username'
     */
    usernameField?: string;
    /**
     * Optional, defaults to 'password'
     */
    passwordField?: string;
}
export interface JWTAuthentication {
    /**
     * Is a REQUIRED string or buffer containing the secret (symmetric)
     * or PEM-encoded public key (asymmetric) for verifying the token's signature.
     */
    secretOrKey: string;
    /**
     * Defines how the JWT token will be extracted from request.
     */
    extractFrom?: JWTRequestExtractor;
    /**
     * If defined the token issuer (iss) will be verified against this value.
     */
    issuer?: string;
    /**
     * If defined, the token audience (aud) will be verified against this value.
     */
    audience?: string;
    /**
     * List of strings with the names of the allowed algorithms. For instance, ["HS256", "HS384"].
     */
    algorithms?: Array<string>;
    /**
     * If true do not validate the expiration of the token.
     */
    ignoreExpiration?: boolean;
    /**
     * Is a function with the parameters verify(request, jwt_payload, done)
     *  - request The user request.
     *  - jwt_payload is an object literal containing the decoded JWT payload.
     *  - done is a passport error first callback accepting arguments done(error, user, info)
     */
    verify?: MiddlewareConfig;
}
export interface JWTRequestExtractor {
    header?: string;
    queryParam?: string;
    authHeader?: string;
    bodyField?: string;
    cookie?: string;
}
export declare let authenticationValidatorSchema: Joi.ObjectSchema;
export declare let apiAuthenticationValidatorSchema: Joi.ObjectSchema;
export declare function validateLocalAuthConfig(config: LocalAuthentication): LocalAuthentication;
export declare function validateBasicAuthConfig(config: BasicAuthentication): BasicAuthentication;
export declare function validateJwtAuthConfig(config: JWTAuthentication): JWTAuthentication;
