import * as Joi from 'joi';
import { AdminConfig } from './admin';
import { AuthenticationConfig } from './authentication';
import { CacheConfig } from './cache';
import { CircuitBreakerConfig } from './circuit-breaker';
import { CorsConfig } from './cors';
import { DatabaseConfig } from './database';
import { AccessLoggerConfig, LoggerConfig } from './logger';
import { MiddlewareConfig } from './middleware';
import { ProtocolConfig } from './protocol';
import { ServiceDiscoveryConfig } from './service-discovery';
import { ThrottlingConfig } from './throttling';
/**
 * The Server config descriptor.
 */
export interface ServerConfig {
    /**
     * Configurations for gateway database (REDIS).
     */
    database: DatabaseConfig;
    /**
     * Folder where the gateway will install its middleware functions.
     */
    middlewarePath?: string;
    /**
     * The root folder where the gateway will work.
     */
    rootPath?: string;
    /**
     * The gateway configuration.
     */
    gateway?: GatewayConfig;
}
/**
 * The Gateway config descriptor.
 */
export interface GatewayConfig {
    /**
     * The gateway protocol configuration
     */
    protocol: ProtocolConfig;
    /**
     * If we are behind a reverse proxy (Heroku, Bluemix, AWS if you use an ELB, custom Nginx setup, etc)
     */
    underProxy?: boolean;
    /**
     * By default, all responses are compressed by the gateway. If you want to disable it set this property to true.
     */
    disableCompression?: boolean;
    /**
     * Disable the validation of API Ids. If the id is not validated, the data could not be synchronizable
     * to TreeGateway dashboard.
     */
    disableApiIdValidation?: boolean;
    /**
     * Configurations for gateway logger.
     */
    logger?: LoggerConfig;
    /**
     * Configurations for gateway access logger.
     */
    accessLogger?: AccessLoggerConfig;
    /**
     * If provided, Configure the admin service for the gateway
     */
    admin?: AdminConfig;
    /**
     * Configure default cors support for API requests. It uses the [cors](https://www.npmjs.com/package/cors) module.
     * It can be configured also in the API configuration
     */
    cors?: CorsConfig;
    /**
     * Configure a timeout for the gateway http.Server. You can inform the amount of milisencods, or use
     * a [human-interval](https://www.npmjs.com/package/human-interval) string. Defaults to 'one minute'.
     */
    timeout?: string | number;
    /**
     * Add filters to the request pipeline. A Filter is a function that receives
     * the request and the response object and must return a boolean value to inform
     * it the given request should target the destination API or if it should be ignored.
     *
     * Example:
     * ```
     * module.exports = function (req) {
     *   return true;
     * };
     * ```
     *
     * Each filter must be defined on its own .js file (placed on middleware/filter folder)
     * and the fileName must match: <filterName>.js.
     *
     * So, the above filter should be saved in a file called myFilter.js and configured as:
     * ```
     * filter:[
     *   { name: "myFilter"}
     * ]
     * ```
     */
    filter?: Array<MiddlewareConfig>;
    /**
     * Configuration for service discovery.
     */
    serviceDiscovery?: ServiceDiscoveryConfig;
    /**
     * Configure an healthcheck endpoint for the gateway. Provide here the path where the
     * healthcheck service will respond.
     */
    healthcheck?: string;
    /**
     * Configure how to handle errors during API pipeline.
     */
    errorHandler?: MiddlewareConfig;
    /**
     * Configure pipeline steps globally, to be imported by api configureations
     */
    config?: ApiPipelineConfig;
    /**
     * Inform how request analytics should be stored by the gateway
     */
    analytics?: RequestAnalyticsConfig;
}
/**
 * Inform how request analytics should be stored by the gateway
 */
export interface RequestAnalyticsConfig {
    /**
     * Enable log recording for the gateway requests
     */
    enabled?: boolean;
    /**
     * The logger middleware
     */
    logger?: MiddlewareConfig;
}
/**
 * Declare common configurations that can be used by different APIs.
 */
export interface ApiPipelineConfig {
    /**
     * Authentication configuration
     */
    authentication?: {
        [index: string]: AuthenticationConfig;
    };
    /**
     * Cache configuration
     */
    cache?: {
        [index: string]: CacheConfig;
    };
    /**
     * CircuitBreaker configuration
     */
    circuitBreaker?: {
        [index: string]: CircuitBreakerConfig;
    };
    /**
     * Cors configuration
     */
    cors?: {
        [index: string]: CorsConfig;
    };
    /**
     * Filter configuration
     */
    filter?: {
        [index: string]: MiddlewareConfig;
    };
    /**
     * Throttling configuration
     */
    throttling?: {
        [index: string]: ThrottlingConfig;
    };
    /**
     * Interceptors configuration
     */
    interceptor?: PipelineInterceptorConfig;
    /**
     * Error Handler configuration
     */
    errorHandler?: {
        [index: string]: MiddlewareConfig;
    };
}
export interface PipelineInterceptorConfig {
    request?: {
        [index: string]: MiddlewareConfig;
    };
    response?: {
        [index: string]: MiddlewareConfig;
    };
}
export declare const requestAnalyticsConfigSchema: Joi.ObjectSchema;
export declare const gatewayConfigValidatorSchema: Joi.ObjectSchema;
export declare let serverConfigValidatorSchema: Joi.ObjectSchema;
export declare function validateGatewayConfig(gatewayConfig: GatewayConfig): Promise<GatewayConfig>;
export declare function validateServerConfig(config: ServerConfig): Promise<ServerConfig>;
