'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const jwt = require("jsonwebtoken");
const _ = require("lodash");
const passport_strategy_1 = require("passport-strategy");
const typescript_ioc_1 = require("typescript-ioc");
const authentication_1 = require("../../../config/authentication");
const middleware_loader_1 = require("../../../utils/middleware-loader");
class JwtExtractors {
    static fromHeader(headerName) {
        return (request) => {
            let token = null;
            if (request.headers[headerName]) {
                token = request.headers[headerName];
            }
            return token;
        };
    }
    static fromBodyField(fieldName) {
        return (request) => {
            let token = null;
            if (request.body && request.body.hasOwnProperty(fieldName)) {
                token = request.body[fieldName];
            }
            return token;
        };
    }
    static fromUrlQueryParameter(paramName) {
        return (request) => {
            let token = null;
            if (request.query.hasOwnProperty(paramName)) {
                token = request.query[paramName];
            }
            return token;
        };
    }
    static fromAuthHeaderWithScheme(authScheme) {
        const authSchemeLower = authScheme.toLowerCase();
        return (request) => {
            let token = null;
            if (request.headers[JwtExtractors.AUTH_HEADER]) {
                const authParams = JwtExtractors.parseAuthHeader(request.headers[JwtExtractors.AUTH_HEADER]);
                if (authParams && authSchemeLower === authParams.scheme.toLowerCase()) {
                    token = authParams.value;
                }
            }
            return token;
        };
    }
    static fromAuthHeaderAsBearerToken() {
        return JwtExtractors.fromAuthHeaderWithScheme(JwtExtractors.BEARER_AUTH_SCHEME);
    }
    static fromCookie(cookieName) {
        return (request) => {
            let token = null;
            cookieName = cookieName ? cookieName : 'token';
            if (request && request.cookies && request.cookies[cookieName]) {
                token = request.cookies[cookieName];
            }
            return token;
        };
    }
    static fromExtractors(extractors) {
        if (!Array.isArray(extractors)) {
            throw new TypeError('extractors.fromExtractors expects an array');
        }
        return (request) => {
            let token = null;
            let index = 0;
            while (!token && index < extractors.length) {
                token = extractors[index](request);
                index++;
            }
            return token;
        };
    }
    static parseAuthHeader(hdrValue) {
        if (_.isArray(hdrValue)) {
            if (hdrValue && hdrValue.length) {
                hdrValue = hdrValue[0];
            }
            else {
                return null;
            }
        }
        const matches = hdrValue.match(JwtExtractors.AUTH_HEADER_REGEX);
        return matches && { scheme: matches[1], value: matches[2] };
    }
}
JwtExtractors.AUTH_HEADER = 'authorization';
JwtExtractors.BEARER_AUTH_SCHEME = 'bearer';
JwtExtractors.AUTH_HEADER_REGEX = /(\S+)\s+(\S+)/;
class JwtStrategy extends passport_strategy_1.Strategy {
    constructor(options) {
        super();
        this.options = {
            algorithms: options.algorithms,
            audience: options.audience,
            ignoreExpiration: !!options.ignoreExpiration,
            issuer: options.issuer,
        };
        this.secretOrKey = options.secretOrKey;
        this.jwtFromRequest = options.jwtFromRequest;
        this.verifyFunction = options.verifyFunction;
    }
    authenticate(req) {
        const token = this.jwtFromRequest(req);
        if (!token) {
            return this.fail({ message: 'No auth token' }, 401);
        }
        jwt.verify(token, this.secretOrKey, this.options, (err, payload) => {
            if (err) {
                return this.fail({ message: 'Invalid token' }, 401);
            }
            else {
                const verified = (error, user) => {
                    if (error) {
                        return this.error(error);
                    }
                    else if (!user) {
                        return this.fail(401);
                    }
                    else {
                        return this.success(user, null);
                    }
                };
                try {
                    this.verifyFunction(req, payload, verified);
                }
                catch (ex) {
                    this.error(ex);
                }
            }
        });
    }
}
module.exports = function (authConfig) {
    authentication_1.validateJwtAuthConfig(authConfig);
    const opts = _.omit(authConfig, 'extractFrom', 'verify');
    if (authConfig.extractFrom) {
        const extractors = _.keys(authConfig.extractFrom);
        if (extractors.length === 1) {
            opts.jwtFromRequest = getExtractor(extractors[0], authConfig.extractFrom[extractors[0]]);
        }
        else {
            const requestExtractors = new Array();
            _.keys(authConfig.extractFrom).forEach(extractor => {
                requestExtractors.push(getExtractor(extractor, authConfig.extractFrom[extractor]));
            });
            opts.jwtFromRequest = JwtExtractors.fromExtractors(requestExtractors);
        }
    }
    else {
        opts.jwtFromRequest = JwtExtractors.fromAuthHeaderAsBearerToken();
    }
    if (authConfig.verify) {
        const middlewareLoader = typescript_ioc_1.Container.get(middleware_loader_1.MiddlewareLoader);
        opts.verifyFunction = middlewareLoader.loadMiddleware('authentication/verify', authConfig.verify);
    }
    else {
        opts.verifyFunction = (req, jwtPayload, done) => {
            done(null, jwtPayload);
        };
    }
    return new JwtStrategy(opts);
};
function getExtractor(extractor, param) {
    switch (extractor) {
        case 'header': return JwtExtractors.fromHeader(param.toLowerCase());
        case 'queryParam': return JwtExtractors.fromUrlQueryParameter(param);
        case 'authHeader': return JwtExtractors.fromAuthHeaderWithScheme(param);
        case 'bodyField': return JwtExtractors.fromBodyField(param);
        case 'cookie': return JwtExtractors.fromCookie(param);
        default:
            return JwtExtractors.fromAuthHeaderAsBearerToken();
    }
}
module.exports.factory = true;
//# sourceMappingURL=jwt.js.map