'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const gateway_1 = require("../../gateway");
const logger_1 = require("../../logger");
const middleware_loader_1 = require("../../utils/middleware-loader");
const time_intervals_1 = require("../../utils/time-intervals");
const Groups = require("../group");
const request_1 = require("../stats/request");
const express_circuit_breaker_1 = require("./express-circuit-breaker");
const redis_state_handler_1 = require("./redis-state-handler");
let ApiCircuitBreaker = class ApiCircuitBreaker {
    constructor() {
        this.activeBreakers = new Map();
    }
    circuitBreaker(apiRouter, api, pipelineConfig) {
        const breakerInfos = new Array();
        const sortedBreakers = this.sortBreakers(api.circuitBreaker, api.path);
        const breakersSize = sortedBreakers.length;
        sortedBreakers.forEach((cbConfig, index) => {
            cbConfig = this.resolveReferences(cbConfig, pipelineConfig);
            const breakerInfo = {};
            const cbStateID = (breakersSize > 1 ? `${api.id}:${index}` : api.id);
            const cbOptions = {
                id: cbStateID,
                maxFailures: (cbConfig.maxFailures || 10),
                rejectMessage: (cbConfig.rejectMessage || 'Service unavailable'),
                rejectStatusCode: (cbConfig.rejectStatusCode || 503),
                stateHandler: new redis_state_handler_1.RedisStateHandler(cbStateID, time_intervals_1.getMilisecondsInterval(cbConfig.resetTimeout, 120000), time_intervals_1.getMilisecondsInterval(cbConfig.timeWindow)),
                timeout: time_intervals_1.getMilisecondsInterval(cbConfig.timeout, 30000),
                timeoutMessage: (cbConfig.timeoutMessage || 'Operation timeout'),
                timeoutStatusCode: (cbConfig.timeoutStatusCode || 504)
            };
            if (this.logger.isDebugEnabled()) {
                this.logger.debug(`Configuring Circuit Breaker for path [${api.path}].`);
            }
            breakerInfo.circuitBreaker = new express_circuit_breaker_1.CircuitBreaker(cbOptions);
            this.configureCircuitBreakerEventListeners(breakerInfo, api.path, cbConfig, api);
            if (cbConfig.group) {
                if (this.logger.isDebugEnabled()) {
                    const groups = Groups.filter(api.group, cbConfig.group);
                    this.logger.debug(`Configuring Group filters for Circuit Breaker on path [${api.path}]. Groups [${JSON.stringify(groups)}]`);
                }
                breakerInfo.groupValidator = Groups.buildGroupAllowFilter(api.group, cbConfig.group);
            }
            breakerInfos.push(breakerInfo);
        });
        this.setupMiddlewares(apiRouter, breakerInfos);
        this.addStopListeners();
    }
    onStateChanged(id, state) {
        const breaker = this.activeBreakers.get(id);
        if (breaker) {
            breaker.onStateChanged(state);
        }
    }
    removeAllBreakers() {
        this.activeBreakers.forEach(breaker => breaker.destroy());
        this.activeBreakers.clear();
    }
    resolveReferences(circuitBreaker, pipelineConfig) {
        if (circuitBreaker.use && pipelineConfig.circuitBreaker) {
            if (pipelineConfig.circuitBreaker[circuitBreaker.use]) {
                circuitBreaker = _.defaults(circuitBreaker, pipelineConfig.circuitBreaker[circuitBreaker.use]);
            }
            else {
                throw new Error(`Invalid reference ${circuitBreaker.use}. There is no configuration for this id.`);
            }
        }
        return circuitBreaker;
    }
    configureCircuitBreakerEventListeners(breakerInfo, path, config, api) {
        if (this.requestLogger.isRequestLogEnabled(api)) {
            breakerInfo.circuitBreaker.on('open', (req) => {
                const requestLog = this.requestLogger.getRequestLog(req);
                if (requestLog) {
                    requestLog.circuitbreaker = 'open';
                }
            });
            breakerInfo.circuitBreaker.on('close', (req) => {
                const requestLog = this.requestLogger.getRequestLog(req);
                if (requestLog) {
                    requestLog.circuitbreaker = 'close';
                }
            });
            breakerInfo.circuitBreaker.on('rejected', (req) => {
                const requestLog = this.requestLogger.getRequestLog(req);
                if (requestLog) {
                    requestLog.circuitbreaker = 'rejected';
                }
            });
        }
        if (config.onOpen) {
            const openHandler = this.middlewareLoader.loadMiddleware('circuitbreaker', config.onOpen);
            breakerInfo.circuitBreaker.on('open', () => {
                openHandler(path, 'open', api.id);
            });
        }
        if (config.onClose) {
            const closeHandler = this.middlewareLoader.loadMiddleware('circuitbreaker', config.onClose);
            breakerInfo.circuitBreaker.on('close', () => {
                closeHandler(path, 'close', api.id);
            });
        }
        if (config.onRejected) {
            const rejectedHandler = this.middlewareLoader.loadMiddleware('circuitbreaker', config.onRejected);
            breakerInfo.circuitBreaker.on('rejected', () => {
                rejectedHandler(path, 'rejected', api.id);
            });
        }
    }
    setupMiddlewares(apiRouter, breakerInfos) {
        breakerInfos.forEach((breakerInfo) => {
            this.activeBreakers.set(breakerInfo.circuitBreaker.id, breakerInfo.circuitBreaker);
            apiRouter.use(this.buildMiddleware(breakerInfo));
        });
    }
    buildMiddleware(breakerInfo) {
        const circuitBreakerMiddleware = breakerInfo.circuitBreaker.middleware();
        return (req, res, next) => {
            if (breakerInfo.groupValidator) {
                if (breakerInfo.groupValidator(req, res)) {
                    circuitBreakerMiddleware(req, res, next);
                }
                else {
                    next();
                }
            }
            else {
                circuitBreakerMiddleware(req, res, next);
            }
        };
    }
    sortBreakers(breakers, path) {
        const generalBreakers = _.filter(breakers, (value) => {
            if (value.group) {
                return true;
            }
            return false;
        });
        if (generalBreakers.length > 1) {
            this.logger.error(`Invalid circuit breaker configuration for api [${path}]. Conflicting configurations for default group`);
            return [];
        }
        if (generalBreakers.length > 0) {
            const index = breakers.indexOf(generalBreakers[0]);
            if (index < breakers.length - 1) {
                const gen = breakers.splice(index, 1);
                breakers.push(gen[0]);
            }
        }
        return breakers;
    }
    addStopListeners() {
        const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
        const stop = () => {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug('API stopped. Removing circuitbreakers handlers.');
            }
            this.removeAllBreakers();
            gateway.removeListener('stop', stop);
            gateway.removeListener('api-reload', stop);
        };
        gateway.on('stop', stop);
        gateway.on('api-reload', stop);
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], ApiCircuitBreaker.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", middleware_loader_1.MiddlewareLoader)
], ApiCircuitBreaker.prototype, "middlewareLoader", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", request_1.RequestLogger)
], ApiCircuitBreaker.prototype, "requestLogger", void 0);
ApiCircuitBreaker = __decorate([
    typescript_ioc_1.AutoWired
], ApiCircuitBreaker);
exports.ApiCircuitBreaker = ApiCircuitBreaker;
//# sourceMappingURL=circuit-breaker.js.map