'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_ioc_1 = require("typescript-ioc");
const events_1 = require("../../config/events");
const database_1 = require("../../database");
const logger_1 = require("../../logger");
const express_circuit_breaker_1 = require("./express-circuit-breaker");
class CircuitBreakerKeys {
}
CircuitBreakerKeys.CIRCUIT_BREAKER_FAILURES = '{circuitbreaker}:failures';
CircuitBreakerKeys.CIRCUIT_BREAKER_STATE = '{circuitbreaker}:state';
exports.CircuitBreakerKeys = CircuitBreakerKeys;
let RedisStateHandler = class RedisStateHandler {
    constructor(id, resetTimeout, timeWindow) {
        this.timerHalfOpen = null;
        this.id = id;
        this.resetTimeout = resetTimeout;
        if (timeWindow) {
            this.timeWindow = timeWindow / 1000;
        }
    }
    initialState() {
        this.database.redisClient.get(this.getRedisStateKey())
            .then((state) => {
            if (state === 'open') {
                this.openState();
            }
            else {
                this.closeState();
            }
        }).catch((err) => {
            this.forceClose();
        });
    }
    isOpen() {
        return this.state === express_circuit_breaker_1.State.OPEN;
    }
    isHalfOpen() {
        return this.state === express_circuit_breaker_1.State.HALF_OPEN;
    }
    isClosed() {
        return this.state === express_circuit_breaker_1.State.CLOSED;
    }
    forceOpen() {
        if (!this.openState()) {
            return false;
        }
        this.notifyCircuitOpen();
        return true;
    }
    forceClose() {
        if (!this.closeState()) {
            return false;
        }
        this.notifyCircuitClose();
        return true;
    }
    forceHalfOpen() {
        if (this.state === express_circuit_breaker_1.State.HALF_OPEN) {
            return false;
        }
        this.state = express_circuit_breaker_1.State.HALF_OPEN;
        this.halfOpenCallPending = false;
        return true;
    }
    incrementFailures() {
        const rdskey = this.getRedisFailuresKey();
        if (this.timeWindow) {
            return new Promise((resolve, reject) => {
                this.database.redisClient.multi()
                    .incr(rdskey)
                    .ttl(rdskey)
                    .exec((err, replies) => {
                    if (err) {
                        return reject(err);
                    }
                    // if this is new or has no expire
                    if (replies[0][1] === 1 || replies[1][1] === -1) {
                        // then expire it after the timeout
                        this.database.redisClient.expire(rdskey, this.timeWindow);
                    }
                    resolve(replies[0][1]);
                });
            });
        }
        else {
            return this.database.redisClient.incr(rdskey);
        }
    }
    onStateChanged(state) {
        switch (state) {
            case 'open':
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug(`Notification received: Circuit for API ${this.id} is open`);
                }
                this.openState();
                break;
            case 'close':
                if (this.logger.isDebugEnabled()) {
                    this.logger.debug(`Notification received: Circuit for API ${this.id} is closed`);
                }
                this.closeState();
                break;
            default:
            // Ignore
        }
    }
    destroy() {
        if (this.timerHalfOpen) {
            clearTimeout(this.timerHalfOpen);
            this.timerHalfOpen = null;
        }
    }
    getRedisFailuresKey() {
        return `${CircuitBreakerKeys.CIRCUIT_BREAKER_FAILURES}:${this.id}`;
    }
    getRedisStateKey() {
        return `${CircuitBreakerKeys.CIRCUIT_BREAKER_STATE}:${this.id}`;
    }
    openState() {
        if (this.state === express_circuit_breaker_1.State.OPEN) {
            return false;
        }
        this.state = express_circuit_breaker_1.State.OPEN;
        // After reset timeout circuit should enter half open state
        this.timerHalfOpen = setTimeout(() => {
            this.timerHalfOpen = null;
            this.forceHalfOpen();
        }, this.resetTimeout);
        return true;
    }
    closeState() {
        if (this.state === express_circuit_breaker_1.State.CLOSED) {
            return false;
        }
        this.halfOpenCallPending = false;
        this.state = express_circuit_breaker_1.State.CLOSED;
        return true;
    }
    notifyCircuitOpen() {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Notifying cluster that circuit for API ${this.id} is open`);
        }
        return this.database.redisClient.multi()
            .set(this.getRedisStateKey(), 'open')
            .publish(events_1.ConfigTopics.CIRCUIT_CHANGED, JSON.stringify({ state: 'open', id: this.id }))
            .exec();
    }
    notifyCircuitClose() {
        if (this.logger.isDebugEnabled()) {
            this.logger.debug(`Notifying cluster that circuit for API ${this.id} is closed`);
        }
        return this.database.redisClient.multi()
            .set(this.getRedisStateKey(), 'close')
            .del(this.getRedisFailuresKey(), this.id)
            .publish(events_1.ConfigTopics.CIRCUIT_CHANGED, JSON.stringify({ state: 'close', id: this.id }))
            .exec();
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", logger_1.Logger)
], RedisStateHandler.prototype, "logger", void 0);
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisStateHandler.prototype, "database", void 0);
RedisStateHandler = __decorate([
    typescript_ioc_1.AutoWired,
    __metadata("design:paramtypes", [String, Number, Number])
], RedisStateHandler);
exports.RedisStateHandler = RedisStateHandler;
//# sourceMappingURL=redis-state-handler.js.map