'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const _ = require("lodash");
const mm = require("micromatch");
const typescript_ioc_1 = require("typescript-ioc");
const errors_1 = require("../../../config/errors");
const gateway_1 = require("../../../gateway");
const logger_1 = require("../../../logger");
const plugin_data_1 = require("../../../service/plugin-data");
const time_intervals_1 = require("../../../utils/time-intervals");
const IPV6_PREFIX = '::ffff:';
const ipFilterConfigSchema = Joi.object().keys({
    blacklist: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
    database: Joi.object().keys({
        checkInterval: Joi.alternatives([Joi.string(), Joi.number().positive()]),
        key: Joi.string()
    }),
    message: Joi.string(),
    statusCode: Joi.number(),
    whitelist: Joi.alternatives([Joi.array().items(Joi.string()), Joi.string()]),
}).xor('blacklist', 'whitelist');
function validateIpFilterConfig(config) {
    const result = Joi.validate(config, ipFilterConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
function getBlacklistFilter(config) {
    const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
    const logger = typescript_ioc_1.Container.get(logger_1.Logger);
    config.blacklist = _.castArray(config.blacklist || []);
    let blocked = config.blacklist;
    if (config.database) {
        const pluginsDataService = typescript_ioc_1.Container.get(plugin_data_1.PluginsDataService);
        pluginsDataService.on('changed', (configKey, data) => {
            blocked = _.union(config.blacklist, data || []);
        });
        const watcher = pluginsDataService.watchConfigurationItems(config.database.key || 'ipFilter:blacklist', time_intervals_1.getMilisecondsInterval(config.database.checkInterval, 30000));
        const stop = () => {
            if (logger.isDebugEnabled()) {
                logger.debug('Gateway stopped. Removing database monitors for ipFilter.');
            }
            pluginsDataService.stopWatchingConfigurationItems(watcher);
            pluginsDataService.removeAllListeners('changed');
            gateway.removeListener('stop', stop);
            gateway.removeListener('api-reload', stop);
        };
        gateway.on('stop', stop);
        gateway.on('api-reload', stop);
    }
    return (req, res) => {
        if (!blocked || !blocked.length) {
            return true;
        }
        let ip = req.ip || req.connection.remoteAddress;
        let allowed = !mm.any(ip, blocked);
        if (allowed && ip.startsWith(IPV6_PREFIX)) {
            ip = ip.substring(IPV6_PREFIX.length);
            allowed = !mm.any(ip, blocked);
        }
        if (!allowed) {
            res.status(config.statusCode || 403).send(config.message || 'Not allowed.');
        }
        return allowed;
    };
}
function getWhitelistFilter(config) {
    const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
    const logger = typescript_ioc_1.Container.get(logger_1.Logger);
    config.whitelist = _.castArray(config.whitelist || []);
    let unblocked = config.whitelist;
    if (config.database) {
        const pluginsDataService = typescript_ioc_1.Container.get(plugin_data_1.PluginsDataService);
        pluginsDataService.on('changed', (configKey, data) => {
            unblocked = _.union(config.whitelist, data || []);
        });
        const watcher = pluginsDataService.watchConfigurationItems(config.database.key || '{ipFilter}:whitelist', time_intervals_1.getMilisecondsInterval(config.database.checkInterval, 30000));
        const stop = () => {
            if (logger.isDebugEnabled()) {
                logger.debug('Gateway stopped. Removing database monitors for ipFilter.');
            }
            pluginsDataService.stopWatchingConfigurationItems(watcher);
            pluginsDataService.removeAllListeners('changed');
            gateway.removeListener('stop', stop);
            gateway.removeListener('api-reload', stop);
        };
        gateway.on('stop', stop);
        gateway.on('api-reload', stop);
    }
    return (req, res) => {
        if (!unblocked) {
            return true;
        }
        let ip = req.ip || req.connection.remoteAddress;
        let allowed = mm.any(ip, unblocked);
        if (!allowed && ip.startsWith(IPV6_PREFIX)) {
            ip = ip.substring(IPV6_PREFIX.length);
            allowed = mm.any(ip, unblocked);
        }
        if (!allowed) {
            res.status(config.statusCode || 403).send(config.message || 'IP not allowed.');
        }
        return allowed;
    };
}
module.exports = function (config) {
    validateIpFilterConfig(config);
    if (config.whitelist) {
        return getWhitelistFilter(config);
    }
    return getBlacklistFilter(config);
};
module.exports.factory = true;
//# sourceMappingURL=ipFilter.js.map