'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const chooser = require("weighted");
const errors_1 = require("../../../config/errors");
const gateway_1 = require("../../../gateway");
const logger_1 = require("../../../logger");
const plugin_data_1 = require("../../../service/plugin-data");
const health_check_1 = require("../../../utils/health-check");
const time_intervals_1 = require("../../../utils/time-intervals");
const errors_2 = require("../../error/errors");
const destinationConfig = Joi.object().keys({
    healthCheck: Joi.string(),
    target: Joi.string().required(),
    weight: Joi.number()
});
const loadBalancerConfigSchema = Joi.object().keys({
    database: Joi.object().keys({
        checkInterval: Joi.alternatives([Joi.string(), Joi.number().positive()]),
        key: Joi.string()
    }),
    destinations: Joi.alternatives([Joi.array().items(destinationConfig), destinationConfig]),
    healthCheckOptions: Joi.object().keys({
        checkInterval: Joi.alternatives([Joi.string(), Joi.number().positive()]),
        failCount: Joi.number(),
        waitTimeout: Joi.alternatives([Joi.string(), Joi.number().positive()])
    }),
    strategy: Joi.string().valid('random', 'round-robin', 'weight')
});
function validateLoadBalancerConfig(config) {
    const result = Joi.validate(config, loadBalancerConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
class Balancer {
    constructor(config) {
        this.fixedServiceInstances = _.castArray(config.destinations || []);
        this.updateInstances(this.fixedServiceInstances, config.healthCheckOptions);
        this.observeDatabase(config);
    }
    observeDatabase(config) {
        if (config.database) {
            const pluginsDataService = typescript_ioc_1.Container.get(plugin_data_1.PluginsDataService);
            const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
            const logger = typescript_ioc_1.Container.get(logger_1.Logger);
            pluginsDataService.on('changed', (configKey, data) => {
                if (!_.isEqual(this.previousDBData, data)) {
                    if (logger.isDebugEnabled()) {
                        logger.debug('New configuration for loadBalancer found on database');
                        logger.debug(data);
                    }
                    this.previousDBData = data;
                    if (data && data.length) {
                        const dbConfig = data.map(item => JSON.parse(item));
                        this.updateInstances(this.mergeInstances(dbConfig), config.healthCheckOptions);
                    }
                    else {
                        this.updateInstances(this.mergeInstances(null), config.healthCheckOptions);
                    }
                }
            });
            const watcher = pluginsDataService.watchConfigurationItems(config.database.key || 'loadBalancer:instances', time_intervals_1.getMilisecondsInterval(config.database.checkInterval, 30000));
            const stop = () => {
                if (logger.isDebugEnabled()) {
                    logger.debug('API stopped. Removing database monitors for loadBalancer.');
                }
                pluginsDataService.stopWatchingConfigurationItems(watcher);
                pluginsDataService.removeAllListeners('changed');
                gateway.removeListener('stop', stop);
                gateway.removeListener('api-reload', stop);
            };
            gateway.on('stop', stop);
            gateway.on('api-reload', stop);
        }
    }
    mergeInstances(data) {
        if (data && data.length) {
            return _.unionWith(this.fixedServiceInstances, data, _.isEqual);
        }
        return this.fixedServiceInstances;
    }
    updateInstances(data, healthCheckOptions) {
        this.serviceInstances = data;
        this.checkInstances(data, healthCheckOptions);
    }
    checkInstances(data, healthCheckOptions) {
        const monitoredInstances = data.filter(destination => destination.healthCheck);
        if (monitoredInstances && monitoredInstances.length) {
            const gateway = typescript_ioc_1.Container.get(gateway_1.Gateway);
            const logger = typescript_ioc_1.Container.get(logger_1.Logger);
            if (this.healthChecker) {
                this.healthChecker.stop();
                this.healthChecker.removeAllListeners();
            }
            else {
                const stop = () => {
                    if (logger.isDebugEnabled()) {
                        logger.debug('API stopped. Removing health checkers for loadBalancer.');
                    }
                    this.healthChecker.stop();
                    this.healthChecker.removeAllListeners();
                    this.healthChecker = null;
                    gateway.removeListener('stop', stop);
                    gateway.removeListener('api-reload', stop);
                };
                gateway.on('stop', stop);
                gateway.on('api-reload', stop);
            }
            const options = healthCheckOptions || {};
            options.servers = monitoredInstances.map(server => server.healthCheck);
            this.healthChecker = new health_check_1.HealthCheck(options);
            this.healthChecker.on('change', (servers) => {
                Object.keys(servers).forEach(server => {
                    const index = _.findIndex(this.serviceInstances, (instance) => instance.healthCheck === server);
                    if (index >= 0) {
                        this.serviceInstances[index].isDown = servers[server].down;
                    }
                });
            });
            this.verifiedInstances = this.serviceInstances.filter(server => !server.isDown);
        }
        else {
            this.verifiedInstances = this.serviceInstances;
        }
    }
}
class RandomBalancer extends Balancer {
    constructor(config) {
        super(config);
    }
    balance() {
        if (this.verifiedInstances.length) {
            return this.verifiedInstances[Math.floor(Math.random() * this.verifiedInstances.length)].target;
        }
        throw new errors_2.UnavailableError(`No instance available.`);
    }
}
class RoundRobinBalancer extends Balancer {
    constructor(config) {
        super(config);
    }
    balance() {
        if (this.verifiedInstances.length) {
            let next = this.verifiedInstances.next || 0;
            const index = next % this.verifiedInstances.length;
            next++;
            this.verifiedInstances.next = next;
            return this.verifiedInstances[index].target;
        }
        throw new errors_2.UnavailableError(`No instance available.`);
    }
}
class WeightedBalancer extends Balancer {
    constructor(config) {
        super(config);
    }
    balance() {
        if (this.instances.length) {
            return chooser.select(this.instances, this.weights);
        }
        throw new errors_2.UnavailableError(`No instance available.`);
    }
    updateInstances(data, healthCheckOptions) {
        super.updateInstances(data, healthCheckOptions);
        this.instances = this.verifiedInstances.map(item => item.target);
        this.weights = this.verifiedInstances.map(item => item.weight);
    }
}
module.exports = function (config) {
    validateLoadBalancerConfig(config);
    let balancer;
    switch (config.strategy) {
        case 'round-robin':
            balancer = new RoundRobinBalancer(config);
            break;
        case 'weight':
            balancer = new WeightedBalancer(config);
            break;
        default:
            balancer = new RandomBalancer(config);
    }
    return (req) => {
        return balancer.balance();
    };
};
module.exports.factory = true;
//# sourceMappingURL=loadBalancer.js.map