'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const Joi = require("joi");
const errors_1 = require("../../../config/errors");
const errors_2 = require("../../error/errors");
const consulConfigSchema = Joi.object().keys({
    clientAgent: Joi.any().required(),
    dc: Joi.string(),
    loadBalancer: Joi.string().valid('random', 'round-robin'),
    serviceName: Joi.string().required(),
    ssl: Joi.boolean(),
    tag: Joi.string()
});
function validateConsulConfig(config) {
    const result = Joi.validate(config, consulConfigSchema);
    if (result.error) {
        throw new errors_1.ValidationError(result.error);
    }
    else {
        return result.value;
    }
}
function random(serviceInstances) {
    return serviceInstances[Math.floor(Math.random() * serviceInstances.length)];
}
function roundRobin(serviceInstances) {
    let next = serviceInstances.next || 0;
    const index = next % serviceInstances.length;
    next++;
    serviceInstances.next = next;
    return serviceInstances[index];
}
function observeService(serviceName, config, knownServiceInstances) {
    const options = {
        passing: true,
        service: serviceName
    };
    if (config.dc) {
        options.dc = config.dc;
    }
    if (config.tag) {
        options.tag = config.tag;
    }
    const watcher = config.clientAgent.watch({
        method: config.clientAgent.health.service,
        options: options
    });
    watcher.on('change', (data) => {
        const serviceInstances = new Array();
        knownServiceInstances.set(serviceName, serviceInstances);
        data.forEach((entry) => {
            serviceInstances.push(`${config.ssl ? 'https' : 'http'}://${entry.Node.Address}:${entry.Service.Port}/`);
        });
    });
}
module.exports = function (config) {
    validateConsulConfig(config);
    const knownServiceInstances = new Map();
    if (config.serviceName) {
        observeService(config.serviceName, config, knownServiceInstances);
    }
    let balance = roundRobin;
    if (config.loadBalancer && config.loadBalancer === 'random') {
        balance = random;
    }
    return (serviceName) => {
        return new Promise((resolve, reject) => {
            if (!serviceName) {
                serviceName = config.serviceName;
            }
            if (!serviceName) {
                return reject(new errors_2.UnavailableError('Service name is missing for consul service discovery middleware.'));
            }
            const serviceInstances = knownServiceInstances.get(serviceName);
            if (!serviceInstances) {
                observeService(serviceName, config, knownServiceInstances);
                config.clientAgent.health.service({
                    service: serviceName
                }, (err, instances) => {
                    if (err) {
                        return reject(err);
                    }
                    if (instances && instances.length) {
                        const entry = instances[0];
                        resolve(`${config.ssl ? 'https' : 'http'}://${entry.Node.Address}:${entry.Service.Port}/`);
                    }
                    else {
                        return reject(new errors_2.UnavailableError(`No instance available for service '${serviceName}'`));
                    }
                });
            }
            else if (serviceInstances.length) {
                resolve(balance(serviceInstances));
            }
            else {
                return reject(new errors_2.UnavailableError(`No instance available for service '${serviceName}'`));
            }
        });
    };
};
module.exports.factory = true;
//# sourceMappingURL=consul.js.map