'use strict';
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const bson_1 = require("bson");
const _ = require("lodash");
const typescript_ioc_1 = require("typescript-ioc");
const errors_1 = require("../../config/errors");
const events_1 = require("../../config/events");
const database_1 = require("../../database");
const errors_2 = require("../../pipeline/error/errors");
const config_1 = require("../../utils/config");
class Constants {
}
Constants.APIS_PREFIX = '{config}:apis';
Constants.ADMIN_API = 'ADMIN_API';
let RedisApiService = class RedisApiService {
    async list(name, version, description, path) {
        let apis = await this.database.redisClient.hgetall(Constants.APIS_PREFIX);
        apis = Object.keys(apis).map((key) => JSON.parse(apis[key]));
        apis = apis.filter((api) => {
            if (name && !api.name.includes(name)) {
                return false;
            }
            if (version && api.version && !`${api.version}`.includes(`${version}`)) {
                return false;
            }
            if (description && api.description && !api.description.includes(description)) {
                return false;
            }
            if (path && api.path && !api.path.includes(path)) {
                return false;
            }
            return true;
        });
        return apis;
    }
    async get(id) {
        const api = await this.database.redisClient.hget(Constants.APIS_PREFIX, id);
        if (!api) {
            throw new errors_2.NotFoundError('Api not found.');
        }
        return JSON.parse(api);
    }
    async create(api) {
        try {
            if (!api.id) {
                api.id = new bson_1.ObjectID().toString();
            }
            this.castArrays(api);
            await this.ensureAPICreateConstraints(api);
            await this.database.redisClient.multi()
                .hmset(`${Constants.APIS_PREFIX}`, api.id, JSON.stringify(api))
                .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: Constants.ADMIN_API }))
                .exec();
            return api.id;
        }
        catch (err) {
            if (typeof err === 'string') {
                err = new errors_1.ValidationError(err);
            }
            throw err;
        }
    }
    async update(api, upsert) {
        try {
            if (upsert && !api.id) {
                api.id = new bson_1.ObjectID().toString();
            }
            this.castArrays(api);
            await this.ensureAPIUpdateConstraints(api, upsert);
            await this.database.redisClient.multi()
                .hmset(`${Constants.APIS_PREFIX}`, api.id, JSON.stringify(api))
                .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: Constants.ADMIN_API, id: api.id }))
                .exec();
        }
        catch (err) {
            if (typeof err === 'string') {
                err = new errors_1.ValidationError(err);
            }
            throw err;
        }
    }
    async remove(id) {
        await this.database.redisClient.multi()
            .hdel(`${Constants.APIS_PREFIX}`, id)
            .publish(events_1.ConfigTopics.CONFIG_UPDATED, JSON.stringify({ packageId: Constants.ADMIN_API }))
            .exec();
    }
    async ensureAPICreateConstraints(api) {
        const exists = await this.database.redisClient.hexists(`${Constants.APIS_PREFIX}`, api.id);
        if (exists) {
            throw new errors_1.ValidationError(`Api ID ${api.id} already exists`);
        }
        let apis = await this.database.redisClient.hgetall(Constants.APIS_PREFIX);
        apis = Object.keys(apis).map((key) => JSON.parse(apis[key]));
        let existingApi = apis.find((a) => a.name === api.name && a.version === api.version);
        if (existingApi) {
            throw new errors_1.ValidationError(`Can not create this api ${api.name}:${api.version} already exists`);
        }
        existingApi = apis.find((a) => a.path === api.path);
        if (existingApi) {
            throw new errors_1.ValidationError(`Can not create this api. Path ${api.path} already registered`);
        }
    }
    async ensureAPIUpdateConstraints(api, upsert) {
        const exists = await this.database.redisClient.hexists(`${Constants.APIS_PREFIX}`, api.id);
        if (!upsert && !exists) {
            throw new errors_2.NotFoundError(`Api not found: ${api.id}.`);
        }
        let apis = await this.database.redisClient.hgetall(Constants.APIS_PREFIX);
        apis = Object.keys(apis).map((key) => JSON.parse(apis[key]));
        let existingApi = apis.find((a) => (a.name === api.name && a.version === api.version) && a.id !== api.id);
        if (existingApi) {
            throw new errors_1.ValidationError(`Can not update this api to ${api.name}:${api.version}. This name conflicts with another existing API (${existingApi.id})`);
        }
        existingApi = apis.find((a) => a.path === api.path && a.id !== api.id);
        if (existingApi) {
            throw new errors_1.ValidationError(`Can not update this api. Path ${api.path} conflicts with another existing API`);
        }
    }
    /**
     * This function cast all array properties inside api configuration to array.
     * It is used to allow user to configure array properties as a single item too.
     * @param api API configuration
     */
    castArrays(api) {
        if (_.has(api, 'group')) {
            config_1.castArray(api, 'group');
            api.group.forEach(config => {
                config_1.castArray(config, 'member');
                config.member.forEach(member => {
                    config_1.castArray(member, 'method');
                    config_1.castArray(member, 'path');
                    config_1.castArray(member, 'protocol');
                });
            });
        }
        config_1.castArray(api, 'proxy.target.allow');
        config_1.castArray(api, 'proxy.target.deny');
        if (_.has(api, 'authentication')) {
            config_1.castArray(api, 'authentication');
            api.authentication.forEach(config => config_1.castArray(config, 'group'));
        }
        if (_.has(api, 'cache')) {
            config_1.castArray(api, 'cache');
            api.cache.forEach(config => {
                config_1.castArray(config, 'group');
                config_1.castArray(config, 'server.preserveHeaders');
            });
        }
        if (_.has(api, 'throttling')) {
            config_1.castArray(api, 'throttling');
            api.throttling.forEach(config => config_1.castArray(config, 'group'));
        }
        if (_.has(api, 'circuitBreaker')) {
            config_1.castArray(api, 'circuitBreaker');
            api.circuitBreaker.forEach(config => config_1.castArray(config, 'group'));
        }
        if (_.has(api, 'cors')) {
            config_1.castArray(api, 'cors');
            api.cors.forEach(config => {
                config_1.castArray(config, 'allowedHeaders');
                config_1.castArray(config, 'exposedHeaders');
                config_1.castArray(config, 'methods');
                config_1.castArray(config, 'group');
            });
        }
        if (_.has(api, 'filter')) {
            config_1.castArray(api, 'filter');
            api.filter.forEach(config => config_1.castArray(config, 'group'));
        }
        if (_.has(api, 'interceptor.request')) {
            config_1.castArray(api, 'interceptor.request');
            api.interceptor.request.forEach(interceptor => config_1.castArray(interceptor, 'group'));
        }
        if (_.has(api, 'interceptor.response')) {
            config_1.castArray(api, 'interceptor.response');
            api.interceptor.response.forEach(interceptor => config_1.castArray(interceptor, 'group'));
        }
    }
};
__decorate([
    typescript_ioc_1.Inject,
    __metadata("design:type", database_1.Database)
], RedisApiService.prototype, "database", void 0);
RedisApiService = __decorate([
    typescript_ioc_1.AutoWired,
    typescript_ioc_1.Singleton
], RedisApiService);
exports.RedisApiService = RedisApiService;
//# sourceMappingURL=api.js.map